import datetime as dt
from enum import Enum
from typing import List, Tuple, Union

import pandas as pd
from pydantic import BaseModel


class Lookback(Enum):
    ONE_HOUR = "1H"
    SIX_HOURS = "6H"
    TWELVE_HOURS = "12H"
    ONE_DAY = "1D"
    ONE_WEEK = "7D"
    ONE_MONTH = "30D"
    THREE_MONTHS = "90D"
    ONE_YEAR = "365D"


class EntityType(Enum):
    FINANCIAL_ASSET = "financial_asset"
    COMPANY = "company"
    ORGANIZATION = "organization"
    PERSON = "person"
    PLACE = "place"


class AssetClass(Enum):
    CRYPTO = "crypto"
    COMMODITIES = "commodities"


class EntityMetricTransform(Enum):
    LAST = 'last'
    AVERAGE = 'average'
    TICKS = 'ticks'
    PRICE_SUMMARY = 'price_summary'


class EntityMetrics(Enum):
    MENTIONS = 'mentions'
    SENTIMENT = 'sentiment'
    EXCITEMENT = 'excitement'
    OPTIMISM = 'optimism'
    PESSIMISM = 'pessimism'
    FEAR = 'fear'
    UNCERTAINTY = 'uncertainty'
    SURPRISE = 'surprise'

class FinancialAssetMetrics(Enum):
    PRICE = 'price'

class SingleMetricTimeSeries(BaseModel):
    metric: Union[EntityMetrics, FinancialAssetMetrics, str]
    values: List[Tuple[Union[str,dt.datetime], float]]

    @property
    def df(self):
        df = pd.DataFrame(self.values, columns=['timestamp', getattr(self.metric, 'value', self.metric)])
        df['timestamp'] = pd.to_datetime(df['timestamp'])
        df.set_index('timestamp', inplace=True)
        return df


class TimeSeriesResponse(BaseModel):
    entity_id: str
    time_series: List[SingleMetricTimeSeries]

class TimeSeriesResample(Enum):
    ONE_MINUTE = '1T'
    FIVE_MINUTES = '5T'
    FIFTEEN_MINUTES = '15T'
    THIRTY_MINUTES = '30T'
    ONE_HOUR = '1H'
    ONE_DAY = '1D'

class TimeHorizon(Enum):
    SHORT = 'short'
    LONG = 'long'



import datetime as dt
import os
import unittest
from rumorz.client import RumorzClient, RumorzAPIException
from rumorz.enums import AssetClass, EntityType, EntityMetrics, Lookback, EntityMetricTransform

rumorz = RumorzClient(api_key=os.environ['RUMORZ_API_KEY'],
                      api_url=os.environ.get('RUMORZ_API_URL', 'http://localhost:8000'))


class TestRumorz(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        entities = rumorz.graph.search_entities(**{
            "name": "Bitcoin",
            "asset_class": AssetClass.CRYPTO,
            "entity_type": EntityType.FINANCIAL_ASSET,
            "symbol_search": "BTC",
            "limit": 1
        })
        assert len(entities) == 1, "Bitcoin entity search returned an unexpected number of results"
        cls.bitcoin_entity_id = entities[0]['id']

    def test_time_series(self):
        timeseries = rumorz.graph.get_metrics(**{
            "operation": EntityMetricTransform.LAST,
            "ids": [self.bitcoin_entity_id],
            "metrics":  [EntityMetrics.SENTIMENT, EntityMetrics.MENTIONS],
            "lookback": Lookback.ONE_DAY,
            "page": 1,
            "limit": 100
        })
        self.assertTrue(len(timeseries) > 0)

    def test_entity_summary(self):
        entity_update = rumorz.agent.summarize(**{
            "id": self.bitcoin_entity_id,
            "timestamp": dt.datetime.utcnow().isoformat(),
            "scores_filter": "sentiment > 0.75"
        })
        self.assertTrue(len(entity_update) > 0)

    def test_get_ranking(self):
        screener = rumorz.graph.get_ranking(**{
            "lookback": Lookback.ONE_DAY,
            "page": 1,
            "limit": 10,
            "sort_by": EntityMetrics.SENTIMENT,
            "entity_type": "financial_asset",
            "scores_filter": 'mentions > 10',
            "ascending": False
        })
        self.assertTrue(len(screener) > 0)

    def test_limit_validation(self):
        try:
            rumorz.graph.get_ranking(**{
                "lookback": "1D",
                "page": 1,
                "limit": 1000,
                "sort_by": "mentions",
                "entity_type": "financial_asset",
            })
        except Exception as e:
            self.assertTrue("limit" in str(e).lower())

    def test_get_feed(self):
        posts = rumorz.graph.get_feed(**{
            "ids": [self.bitcoin_entity_id],
            "lookback": "3D",
            "scores_filter": "sentiment > 0.75",
            "page": 1,
            "limit": 10
        })
        self.assertTrue(len(posts) > 0)

    def test_get_price_stats(self):
        summary = rumorz.graph.get_price_stats(**{
            "id": self.bitcoin_entity_id,
        })
        print(summary)
        self.assertTrue(len(summary) > 0)



# <start_ignore>
    def test_agent_state(self):
        try:
            rumorz.agent.get_state(id='xxxx')
        except Exception as e:
            self.assertTrue(isinstance(e, RumorzAPIException))

    def test_agent_logs(self):
        logs = rumorz.agent.get_logs(id='36a5e6ba-160d-45bf-b28a-bea369e0136f',
                                     lookback="1D",
                                     limit=10)
        self.assertIsNotNone(logs)
# <end_ignore>


import datetime as dt
import os
from datetime import datetime, timedelta

import pandas as pd
import streamlit as st
from pydantic import BaseModel
from streamlit_autorefresh import st_autorefresh

from rumorz.client import RumorzClient
from smartpy.utility import dt_util



rumorz = RumorzClient(api_url="http://localhost:8000")

import re

def remove_enum_mentions(input: str) -> str:
    return re.sub(r'(\w+)_([a-z])', lambda m: f"{m.group(1)} {m.group(2)}", input)

def get_log_content(log):
    ignore = ['FinishedCurrentTask']
    tools_emojis = {
        "Reasoning": "🤔",
        "UpdatePlan": "✅",
        "AlertUser": "👋",
        "Trader": "🗣️",
        "UpdateWatchlist": "📋",
        "UpdateMemory": "💾",
        "Sleep": "😴",
        "GetRealTimeUpdate": "🌐",
    }
    tool_descriptions = {
        "Reasoning": "Thinking",
        "AlertUser": "Alerting user",
        "UpdatePlan": "Ready to work",
        "Trader": "Trading desk",
        "UpdateWatchlist": "Updating watchlist",
        "UpdateMemory": "Updating memory",
        "Sleep": "Waiting...",
        "FinishedCurrentTask": "Subtask completed",
        "GetRealTimeUpdate": "Querying Rumorz Graph",
    }

    content = None
    match log['tool_name']:
        case 'GetRealTimeUpdate':
            content = f"Fetching real-time updates for {log['arguments']}"
        case 'Reasoning':
            content = f"{log['arguments']['reasoning']}"
        case 'UpdatePlan':
            content = f""
        case 'UpdateMemory':
            content = f"updated {log['arguments']['key']}"
        case 'Sleep':
            content = f"{log['arguments']['message']}"
        case 'AlertUser':
            content = f"{log['arguments']['message']}"
        case 'Trader':
            content = f"message to the desk: {log['arguments']['message']}"
        case 'SearchEntities':
            content = f"Searching entities: {log['arguments']}"
        case 'GetTokensSocialRanking':
            arguments = ", ".join([f"{k}: {v}" for k, v in log['arguments'].items()])
            content = f"Fetching Graph ranking by : {arguments}"
        case 'UpdateWatchlist':
            content = f"Updating Feed with: {log['arguments']}"
        case 'GetEntitySummaryUpdate':
            content = f"Entity summary update: {log['arguments']}"
        case 'GetFeed':
            content = f"Entity Feed: {log['arguments']}"
        case 'GetPriceSummary':
            content = f"Fetched price summary fpr {log['arguments']}"
        case 'SendEmail':
            content = "Sending email to your inbox"

    if content is None:
        content = f""


    content = content.replace('the missing fields', 'my memory',)
    content = content.replace('memory.', '')



    if log['tool_name'] in ignore:
        return None

    return {
        "timestamp": log["timestamp"],
        "content": f'<b style="color:white;">[ {tools_emojis.get(log["tool_name"], "🔧")} {tool_descriptions.get(log["tool_name"], log["tool_name"])} ]</b> {content}'
    }


st.set_page_config(layout="wide")
st_autorefresh(interval=5000, key="data_refresh")

if 'last_agent_refresh_timestamp' not in st.session_state:
    st.session_state.last_agent_refresh_timestamp = (datetime.utcnow() - timedelta(days=19)).isoformat()
if 'logs' not in st.session_state:
    st.session_state.logs = []
if 'page' not in st.session_state:
    st.session_state.page = 1

st.markdown(
    """
    <style>
    body {
        background-color: black;
        color: gray;
        margin-top: -50px;
    }
    .header {
        display: flex;
        align-items: center;
        justify-content: space-between;
        padding: 20px;
        border: 2px solid #1d222b;
        margin-bottom:30px;
        border-radius: 10px;
    }
    .no-border {
        border: none!important;
    }
    .header img {
        border-radius: 50%;
        width: 80px;
        height: 80px;
    }
    .header .info {
        flex-grow: 1;
        margin-left: 20px;
    }
    .header .info h1 {
        margin: 0;
        font-size: 24px;
        color: white;
    }
    .header .info p {
        margin: 5px 0;
        font-size: 16px;
    }
    h1, h2, h3, h4, h5, h6 {
        color: white;
    }
    .log-box {
        border: 1px solid #1d222b;
        border-radius: 10px;
        padding: 10px;
        margin-bottom: 10px;
        word-wrap: break-word;
        color:gray;
    }
    .divider {
        border-top: 2px solid 1d222b;
        margin: 10px 0;
    }
    .scrollable-container {
        max-height: 400px;
        overflow-y: auto;
    }
    .padded-container {
        padding: 25px;
    }
    </style>
    """,
    unsafe_allow_html=True
)

st.markdown(
    """
    <div class="header no-border">
        <img src="https://i.postimg.cc/s2P93CGN/blue-logo-no-text-svg.png" alt="Logo" />
        <div class="info">
            <h4>rumorz.io</h4>
        </div>
    </div>
    """,
    unsafe_allow_html=True
)

agent_id = 'fcbae6ae-acd9-4cfe-8a9e-cfbc1fe669a7'
st.session_state.agent_id = agent_id


def format_tool_calls(task):
    markdowns = []
    for tool_call in task['tool_calls']:
        markdowns.append(f"[{tool_call['tool_name'].replace('functions.','')}] {task['title']}\n")
    return markdowns


if 'agent_id' in st.session_state:
    data = rumorz.agent.get_state(id=st.session_state.agent_id)
    new_logs = rumorz.agent.get_logs(
        id=st.session_state.agent_id,
        from_timestamp=st.session_state.last_agent_refresh_timestamp,
        to_timestamp=datetime.utcnow().isoformat(),
        limit=10,
        page=1
    )
    st.session_state.logs = [get_log_content(log) for log in new_logs if log] + st.session_state.logs
    st.session_state.last_agent_refresh_timestamp = datetime.utcnow().isoformat()

    brain = data['brain']
    memory = brain.get("memory", {})
    positioning = memory.get("positioning", "")
    portfolio = brain.get("portfolio_summary", {})
    watch_list = brain.get("watch_list", {}).get("entities", [])
    session_start = brain.get("session_start", "N/A")
    session_duration = brain.get("session_duration", "N/A")
    risk_tolerance = memory.get("risk_tolerance", "N/A")
    initial_goal = brain.get("initial_goal", "N/A")
    name = data.get("name", "N/A")
    personality = brain.get("personality", "N/A")
    plan = brain.get("plan", {}).get("tasks", [])

    st.markdown(
        f"""
        <div class="header">
            <img src="https://i.postimg.cc/wxbw3H3t/kai-avatar.png" alt="Kai" className="w-12 h-12 rounded-full mr-4" />
            <div class="info"> <h1>{name}</h1>
        <p>Function: <span style="color:gray;">Portfolio Manager</span></p>
        <p>Asset class: <span style="color:gray;">Digital Assets</span></p>
        <p>Coverage: <span style="color:gray;">Bitcoin</span></p>
        </div>
        """.format(risk_tolerance),
        unsafe_allow_html=True
    )

    st.markdown('<div class="divider"></div>', unsafe_allow_html=True)

    st.markdown('<div class="padded-container">', unsafe_allow_html=True)
    col1, col2 = st.columns([1, 1])
    with col1:
        st.subheader("Activity")
        st.markdown('<div class="divider"></div>', unsafe_allow_html=True)
        with st.container():
            st.markdown('<div class="scrollable-container">', unsafe_allow_html=True)
            for log in st.session_state.logs:
                if log:
                    st.markdown(f'<div class="log-box">{log["content"]}</div>', unsafe_allow_html=True)
            st.markdown('</div>', unsafe_allow_html=True)
        if st.button("Load more"):
            older_logs = rumorz.agent.get_logs(
                id=st.session_state.agent_id,
                from_timestamp=str(dt_util.toDatetime(st.session_state.logs[-1]['timestamp']) - dt.timedelta(days=10)),
                to_timestamp=str(st.session_state.logs[-1]['timestamp']),
                limit=10,
                page=st.session_state.page * 10
            )
            st.session_state.logs.extend([get_log_content(log) for log in older_logs])
            st.session_state.page += 1
            st.rerun()
    with col2:
        tabs = st.tabs(["Market summary", "Portfolio", "Profile"])

        with tabs[0]:
            st.write(memory.get("market_conditions", ""))

        with tabs[1]:
            if type(portfolio) == dict:
                st.subheader("Trades")
                if 'trades' in portfolio:
                    st.write(pd.DataFrame(portfolio['trades']).to_html(index=False), unsafe_allow_html=True)
                st.subheader("Summary")
                if 'summary' in portfolio:
                    df = pd.DataFrame(list(portfolio['summary'].items()), columns=['stat', 'value'])
                    st.write(df.to_html(index=False), unsafe_allow_html=True)

            else:
                st.write("No trades yet")

        with tabs[2]:
            st.write(f"- **Initial goal**:  50% monthly return")
            st.write(f"- **Starting capital**: $100k")
            st.write(f"- **Risk tolerance**: {risk_tolerance.lower().capitalize() if risk_tolerance else ''}")


    st.markdown('</div>', unsafe_allow_html=True)



import json
from enum import Enum


class RumorzJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Enum):
            return obj.value
        if isinstance(obj, dict):
            return {key: self.default(value) for key, value in obj.items()}
        if isinstance(obj, list):
            return [self.default(element) for element in obj]
        return super().default(obj)



import importlib.util
import json
import subprocess
import traceback
from textwrap import dedent

from litellm import completion
from pydantic import BaseModel, Field
from smartpy.utility import os_util


class RumorzCopilot:

    def __init__(self,
                 model):
        self.model = model

    def load_sdk_source_code(self, include_client=True, include_enums=True, include_tests=True):
        package_name = "rumorz"
        spec = importlib.util.find_spec(package_name)
        if spec and spec.origin:
            sdk_path = spec.origin.replace('__init__.py', '')
        else:
            raise Exception(f"Package {package_name} not found")

        client_content = enums_content = test_content = ""

        if include_client:
            client_path = sdk_path + 'client.py'
            with open(client_path, 'r') as file:
                client_content = f"<RUMORZ SDK SOURCE CODE>\nfile_path: {client_path}\n{file.read()}\n"

        if include_enums:
            enums_path = sdk_path + 'enums.py'
            with open(enums_path, 'r') as file:
                enums_content = f"<ENUMS>\nfile_path: {enums_path}\n{file.read()}\n"

        if include_tests:
            test_path = sdk_path + 'tests/test_sdk.py'
            with open(test_path, 'r') as file:
                test_content = f"<TESTS>\nfile_path: {test_path}\n{file.read()}\n"

        docs = dedent(f"{client_content}{enums_content}{test_content}")
        return docs


    def run_script(self, script_path):
        result = subprocess.run(['python', script_path], capture_output=True, text=True)
        return result

    def create_file(self,
                    file_description,
                    output_path,
                    include_client=True,
                    include_enums=True,
                    include_tests=True):

        source_code = self.load_sdk_source_code(include_client, include_enums, include_tests)

        class SDKCopilotResponse(BaseModel):
            reasoning: str = Field(...,
                                   description="Write a complete reasoning and planning on requirements and steps to write the script based on the documentation and the provided script description")
            file_content: str = Field(..., description="The content of the file to create")


        response = completion(
            model=self.model,
            response_format=SDKCopilotResponse,
            messages=[
                {"role": "system", "content": f"""
ROLE:
You are an expert in Programming and the Rumorz Python SDK. Rumorz provides Financial markets AI intelligence through
the Graph and an Agent. The Graph gives access to entities (tokens, people, companies, etc.) and their posts, sentiment
etc. The Agent provides summaries and insights on entities. 

TASK
You are tasked to create a fully working Python script based on user input. The script must use the Rumorz Python SDK. 
Your output will be in json

GUIDELINES
- Minimize the number of API queries.
- If you hesitate, tell the user to ask again with specific improvements to make in his query
- Your script should always print a success message with details at the end of the execution

DOCUMENTATION
Use the following documentation and Enums to build working queries:
<SOURCE CODE>
{source_code}
<SOURCE CODE>
"""},
                {"role": "user",
                 "content": f"""The user message to describe the script required:
Description:\n{file_description}
"""
                 }
            ],

        )
        class RumorzCopilotException(Exception):
            pass

        try:
            script_code = json.loads(response.choices[0].message.content)['file_content']
            script_code = script_code.replace('```yaml', '').replace('```', '')
            with open(output_path, 'w') as f:
                f.write(script_code)
        except Exception as e:
            raise RumorzCopilotException("Error while parsing LLM response ", traceback.format_exc())



import json
import os

import litellm
import pandas as pd
import requests

from rumorz.encoder import RumorzJSONEncoder

litellm.set_verbose = False


class RumorzAPIException(Exception):
    pass


class RumorzClient:
    def __init__(self,
                 api_key=os.environ['RUMORZ_API_KEY'],
                 api_url=os.environ.get('RUMORZ_API_URL', 'http://rumorz-api.eastus2.azurecontainer.io'),
                 api_version='v0'):
        self.api_url = api_url
        self.api_version = api_version
        self.headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json',
            'X-API-Key': api_key
        }
        self.graph = self.Graph(self)
        self.agent = self.Agent(self)
        response = requests.post(self.api_url + '/ping', headers=self.headers, timeout=5)
        response.raise_for_status()

    def call_function(self, function_name, params):
        url = f"{self.api_url}/{self.api_version}/{function_name}"
        json_data = json.dumps(params, cls=RumorzJSONEncoder)
        response = requests.post(url, json=json_data, headers=self.headers, timeout=30)
        try:
            response.raise_for_status()
            return response.json()['data']
        except Exception as e:
            raise RumorzAPIException(response.text)

    class Graph:

        def __init__(self, api):
            self.api = api

        def get_feed(self,
                     **kwargs):
            return self.api.call_function('graph/get_feed', params=kwargs)

        def search_entities(self,
                            **kwargs):
            """
            Search for entities in the Rumorz Graph by name and/or symbol
            """
            return self.api.call_function('graph/search_entities', params=kwargs)

        def get_ranking(self,
                        **kwargs):
            """
            Returns a ranking of every single entity in the Graph based on the given parameters.
            """
            return self.api.call_function('graph/get_ranking', params=kwargs)

        def get_metrics(self,
                        as_df=True,
                        **kwargs):
            """
            Get metrics for an entity in the Rumorz Graph
            """
            timeseries = self.api.call_function('graph/get_metrics', params=kwargs)
            if as_df:
                dfs = {}
                for node_ts in timeseries:
                    data = {ts['metric']: [v[1] for v in ts['values']] for ts in node_ts['time_series']}
                    timestamps = [v[0] for v in node_ts['time_series'][0]['values']]
                    df = pd.DataFrame(data, index=pd.to_datetime(timestamps))
                    dfs[node_ts['entity_id']] = df
                return dfs

        def get_price_stats(self,
                            **kwargs):
            """
            Get summary price stats of a financial asset entity in the Rumorz Graph
            """
            return self.api.call_function('graph/get_price_stats', params=kwargs)


    class Agent:
        def __init__(self, api):
            self.api = api

        def create_agent(self,
                         **kwargs):
            """
            Create an agent with a goal, name and personality
            """
            return self.api.call_function('agent/create_agent', params=kwargs)

        def summarize(self,
                      **kwargs):
            """
            Summarize events and news for any entity in the graph
            """
            return self.api.call_function('agent/summarize', params=kwargs)

        def get_state(self,
                      **kwargs):
            """
            Get the state of an agent. This includes current sentiment, personality, trading strategy, and more.
            """
            return self.api.call_function('agent/get_state', params=kwargs)

        def get_logs(self,
                     **kwargs):
            return self.api.call_function('agent/get_logs', params=kwargs)
